/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

#ifndef GAMECONSTANTS_H
#define GAMECONSTANTS_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

//
// The scale of the simulation
//

const double kScale = 55;   // pixels / metre

//
// Other constants
//

const double kPi = 3.14159265358979323846;
const double kTwoPi = 2.0 * kPi;
const double kInvTwoPi = 1.0 / kTwoPi;
const double kPiOn2 = 0.5 * kPi;

///--------------------------------------------------------------------------------------
//  sign_smooth
//
//  This utility function smooths the discontinuity in the sign function which
//  is very helpfull when you want to be solving an ODE containing such things.
///--------------------------------------------------------------------------------------

inline double sign_smooth(double x, double k = 1.0)
{
	if (-k < x && x < k)
		return x / k;
	
	if (x > 0.0)
		return 1.0;
	
	// x < 0.0
	return -1.0;
}

///--------------------------------------------------------------------------------------
//  reduce_by_at_most
//
//  This utility function adds or subtracts the value in r from val making it closer to
//  zero but never changing its sign. If |val| < r then it sets val equal to zero.
///--------------------------------------------------------------------------------------

inline void reduce_by_at_most(double& val, double r)
{
	if (val > r)
		val -= r;
	else if (val < -r)
		val += r;
	else
		val = 0.0;
}

#endif
