/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Zilibowitz on 23/02/05.
 *
 *  This is intended to be portable code for returning the time
 *  in seconds. Preferably using microsecond accuracy or better.
 *
 */

#if defined(__APPLE__) && defined(__MACH__)

// Microsecond accuracy timing for Mac OS X

#include </Developer/Headers/FlatCarbon/Timer.h>

double GetSeconds()
{
	unsigned long long t;
	Microseconds((UnsignedWide*)(&t));
	
	// use this to check if UnsignedWide is implemented as little endian
	#if TARGET_RT_LITTLE_ENDIAN
		t = Endian64_Swap(t);
	#endif
	
	return t*1e-6;
}

#else

/*
note: "The resolution of the system clock is hardware dependent,
and the time may be updated continuously or in ``ticks.''" - from the man
page for gettimeofday. Hopefully the system uses microsecond resolution. If
the system uses ticks then this could cause serious problems. In such cases
some alternative method will have to be used.
*/

#include <sys/time.h>

double GetSeconds()
{
    struct timeval t;
    gettimeofday(&t, NULL);
    t.tv_sec -= 1133730000l;
    return t.tv_sec + (t.tv_usec * 1e-6);
}

#endif
