/*
 *  HotPotatoGame.cpp
 *  ToyCars
 *
 *  Created by Ruben Zilibowitz on 17/12/05.
 *  Copyright 2005 Ruben Zilibowitz. All rights reserved.
 *
 */

#include "HotPotatoGame.h"
#include "GameConstants.h"
#include "glf.h"
#include "TcPreferences.h"

extern TcPreferences *gPreferences;

HotPotatoGame::HotPotatoGame(ScTilemap* inTilemap, Player** inPlayers, int inNumPlayers, Tuple inStartLineA, Tuple inStartLineB, double inFuseTime, const list<Tuple> *inPathPolygon)
    :   CarGame(inTilemap, inPlayers, inNumPlayers, inStartLineA, inStartLineB, inPathPolygon),
        bombFuseLength(inFuseTime)
{
    // load and create bomb
    playerWithBomb = players[0];
    bomb = new PlayerBomb(bombFuseLength, playerWithBomb, this, &animation);
    animation.push_back(bomb);
}

HotPotatoGame::~HotPotatoGame()
{
}

void HotPotatoGame::renderEveryTexture()
{
    int i;
    CarGame::renderEveryTexture();
        // render all the bomb sprite frames
    bomb->moveTo(0,0);
    for (i = 0; i < bomb->getNumFrames(); i++)
    {
        bomb->setFrame(i);
        bomb->doDraw();
    }
}

void HotPotatoGame::collisionAction(list<Contact> &contacts, Player* playerI, Player* playerJ)
{
    // check if the collision is hard enough to switch the bomb between cars
    if (-contacts.back().compute_velocity() > 4.0)
    {
        if (playerWithBomb == playerI)
            playerWithBomb = playerJ;
        else if (playerWithBomb == playerJ)
            playerWithBomb = playerI;
        bomb->follow(playerWithBomb);
    }
}

void HotPotatoGame::drawTime(double theTime)
{
    CarGame::drawTime(bombFuseLength - theTime);
}

void HotPotatoGame::detonate(Player* player)
{
	Tuple vec;
	
	playersInGame.remove(player);
	player->car.setVisible(false);
	
	vec = player->car.getLocation_wc() * kScale;
	for (int i = 0; i < 2200; i++)
		particles->newParticle(Particle(vec.x, vec.y, 8.0, (drand48()-0.5)*20.0, (drand48()-0.5)*20.0, kScale));
	
	for (list<Player*>::iterator i = playersInGame.begin(); i != playersInGame.end(); i++)
	{
		// reciprocal of distance
		vec = (*i)->car.getLocation_wc() - player->car.getLocation_wc();
		vec = (40.0/((vec*vec))) * vec;
		(*i)->car.applyImpulse(vec);
	}
}

void HotPotatoGame::drawDisplay(Player *player)
{
	drawSpeedometer(player);
	drawRPMMeter(player);
	if (runningTime > bombFuseLength) {
		if (bomb->getFollowPlayer() == player) {
			drawLose(player);
			//printf("%x loses\n", player);
		}
		else {
			drawWin(player);
			//printf("%x wins\n", player);
		}
	}
}

void HotPotatoGame::drawLose(Player *player)
{
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glColor4f(1.0f,1.0f,1.0f,0.5f);
	glPushMatrix();
	glScalef(12, 12, 1);
	glTranslatef(2, gPreferences->getScreenHeight()/24.0 + 1, 0);
	glfDrawSolidString("You lose");
	glPopMatrix();
}

void HotPotatoGame::drawWin(Player *player)
{
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glColor4f(1.0f,1.0f,1.0f,0.5f);
	glPushMatrix();
	glScalef(12, 12, 1);
	glTranslatef(2, gPreferences->getScreenHeight()/24.0 + 1, 0);
	glfDrawSolidString("You win");
	glPopMatrix();
}
