/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Zilibowitz on 14/02/05.
 */

#include "Kaboom.h"
#include <cmath>

Kaboom::Kaboom(double inXLoc, double inYLoc, double inLifeSpan, ScAnimation *inAnimation)
	: ScPanel(4, 4), innerRadius(1), outerRadius(2), xLoc(inXLoc), yLoc(inYLoc), lifeSpan(inLifeSpan), animation(inAnimation)
{
	centreAt(xLoc, yLoc);
}

void Kaboom::doDraw()
{
	int detail = std::min(int(outerRadius*0.95), 100);
	double dA = M_PI*2.0/detail;
	double A = 0.0;
	double color = 1.0 - 1.0/(0.00006*innerRadius*innerRadius + 1.0);
	
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glColor4d(1.0, color, color, 1-color);
	glBegin(GL_TRIANGLE_STRIP);
	
	for (int i = 0; i < detail; i++)
	{
		glVertex2d(innerRadius*cos(A) + xLoc, innerRadius*sin(A) + yLoc);
		glVertex2d(outerRadius*cos(A) + xLoc, outerRadius*sin(A) + yLoc);
		A += dA;
		// nb: perform an optimisation here - precompute e^(i*dA) as rotation transform etc...
	}
	glVertex2d(innerRadius + xLoc, yLoc);
	glVertex2d(outerRadius + xLoc, yLoc);
	
	glEnd();
}

void Kaboom::process(double dt)
{
	innerRadius += dt * 40000.0 / (innerRadius + 40.0);
	outerRadius = 1.41*innerRadius;
	
	bounds.left = xLoc - outerRadius;
	bounds.bottom = yLoc - outerRadius;
	bounds.right = xLoc + outerRadius;
	bounds.top = yLoc + outerRadius;
	
	lifeSpan -= dt;
    
	if (lifeSpan < 0.0) {
        animation->scheduleRemove(this);
        //animation->remove(this);
		//delete this;
    }
}
