/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Zilibowitz on 8/04/05.
 */

#include "Minimap.h"
#include "GameConstants.h"
#include "TcPreferences.h"

extern TcPreferences *gPreferences;

Minimap::Minimap(const ScTilemap& tilemap, list<Player*>& inPlayersList)
	: playersList(inPlayersList), imageWidth(tilemap.getMapWidth()), imageHeight(tilemap.getMapHeight()),
		xscale(1.0 / tilemap.getTileWidth()), yscale(1.0 / tilemap.getTileHeight())
{
	int i;
	
	// setup color table
	
	GLfloat rvals[32];
	GLfloat gvals[32];
	GLfloat bvals[32];
	GLfloat avals[32];
	for (i = 0; i < 32; i++)
	{
		rvals[i] = 1.0;
		gvals[i] = 1.0;
		bvals[i] = 1.0;
		avals[i] = 0.2;
	} avals[25] = 0.6;
	glPixelMapfv(GL_PIXEL_MAP_I_TO_R, 32, rvals);
	glPixelMapfv(GL_PIXEL_MAP_I_TO_G, 32, gvals);
	glPixelMapfv(GL_PIXEL_MAP_I_TO_B, 32, bvals);
	glPixelMapfv(GL_PIXEL_MAP_I_TO_A, 32, avals);
	glPixelTransferi(GL_MAP_COLOR, 1);
	
	// load minimap
	
	ScPixelData pix = {tilemap.getMapWidth(), 1, GL_RGBA, tilemap.getMapWidth(), tilemap.getMapHeight(), GL_COLOR_INDEX, GL_UNSIGNED_BYTE, (const char *)tilemap.getMapData()};
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	minimapImage = new ScImage(0, pix, false);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
	glPixelTransferi(GL_MAP_COLOR, 0);
}

Minimap::~Minimap()
{
	if (minimapImage)
		delete minimapImage;
}

void Minimap::draw(double where_x, double where_y, double scale)
{
	// draw a minimap
	
	/*
	const double ms = 1.0;
	const double mx = 100;
	const double my = gPreferences->getScreenHeight() - tilemap.getMapHeight()*scale - 20;
	*/
	
	where_y = gPreferences->getScreenHeight() - imageHeight*scale - where_y;
	
	glColor4ub(255, 255, 255, 255);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glLoadIdentity();
	glOrtho(0, gPreferences->getScreenWidth(), 0, gPreferences->getScreenHeight(), -1, 1);
	glTranslatef(where_x, where_y, 0);
	glScalef(scale, scale, 1);
	minimapImage->draw(0, 0);
	
	// draw players in game into minimap
	
	glDisable(GL_TEXTURE_2D);
	glColor4ub(255, 255, 255, 255);
	glLoadIdentity();
	glOrtho(0, gPreferences->getScreenWidth(), 0, gPreferences->getScreenHeight(), -1, 1);
	glTranslatef(where_x, where_y, 0);
	glScaled(scale*xscale, scale*yscale, 1);
	glBegin(GL_QUADS);
	for (list<Player*>::iterator i = playersList.begin(); i != playersList.end(); i++)
	{
		glVertex2d((*i)->car.getBounds().left, (*i)->car.getBounds().top);
		glVertex2d((*i)->car.getBounds().left, (*i)->car.getBounds().bottom);
		glVertex2d((*i)->car.getBounds().right, (*i)->car.getBounds().bottom);
		glVertex2d((*i)->car.getBounds().right, (*i)->car.getBounds().top);
	}
	glEnd();
}
