/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben on Wed Aug 25 2004.
 */

#ifndef CONVEX_H
#define CONVEX_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <list>
using namespace std;

#include "Contact.h"

// Convex class
// This must be a covex polygon stored as vertices in counter-clockwise order.
// The parent rigid body object may be set or left as NULL to indicate not
// beonging to any rigid body.

class Convex : public list<Tuple>
{
public:
    Convex() : parent(NULL) {}
    Convex(RigidBody* inParent) : parent(inParent) {}
    Convex(const Tuple* array, int size);
    
    iterator last() { return --iterator(end()); }
    const_iterator last() const { return --const_iterator(end()); }
    
    iterator next(iterator i) { i++; if (i == end()) i++; return i; }
    const_iterator next(const_iterator i) const { i++; if (i == end()) i++; return i; }
    
    iterator prev(iterator i) { i--; if (i == end()) i--; return i; }
    const_iterator prev(const_iterator i) const { i--; if (i == end()) i--; return i; }
    
    void setParent(RigidBody* inParent) { parent = inParent; }
    RigidBody* getParent() { return parent; }
    
    static bool collide(const Convex& geomA, const Convex& geomB, list<Contact>& contacts);
    
protected:
    Convex::const_iterator deepest_point(const Tuple& N) const;
    
    RigidBody* parent;
};

#endif
