/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben on Sun Aug 29 2004.
 */

#include "Geom.h"

Geom::Geom(const Convex* array, int size)
{
    for (int n = 0; n < size; n++)
        push_back(array[n]);
}

Geom::Geom(Convex** arrayPtrs, int size)
{
    for (int n = 0; n < size; n++)
        push_back(*arrayPtrs[n]);
}

void Geom::setParent(RigidBody* inParent)
{
    for (iterator i = begin(); i != end(); i++)
    {
        i->setParent(inParent);
    }
}

RigidBody* Geom::getParent()
{
    if (empty())
        return NULL;
    else
        return begin()->getParent();
}

bool Geom::collide(const Geom& geomA, const Geom& geomB, list<Contact>& contacts)
{
    bool collided = false;
    const_iterator ia, ib;
    
    for (ia = geomA.begin(); ia != geomA.end(); ia++)
    {
        for (ib = geomB.begin(); ib != geomB.end(); ib++)
        {
            if (Convex::collide(*ia, *ib, contacts))
                collided = true;
        }
    }
    
    return collided;
}

bool Geom::collide(const Geom& geomA, const Convex& geomB, list<Contact>& contacts)
{
    bool collided = false;
    const_iterator ia, ib;
    
    for (ia = geomA.begin(); ia != geomA.end(); ia++)
    {
        if (Convex::collide(*ia, geomB, contacts))
            collided = true;
    }
    
    return collided;
}
