/*
 *  RobotPlayer.cpp
 *  toycars
 *
 *  Created by Ruben Zilibowitz on 5/08/06.
 *  Copyright 2006 Ruben Zilibowitz. All rights reserved.
 *
 */

#include "RobotPlayer.h"

RobotPlayer::RobotPlayer(ToyCar& inCar, list<Tuple> *path, int inPlayerNumber)
	: Player(inCar, path, inPlayerNumber), stuckTime(0.0)
{
}

void RobotPlayer::integrateKeys(double dt)
{	
	updatePathLocation();
	
	if (stuckTime > 1.5 && pathClosestPerp > 0.1) {
		// Try to get unstuck by driving towards the path
		Tuple aim = 0.4*pathClosestPoint + 0.6*(*pathB);
		car.mBrake = 0.0;
		double angle = atan2(aim.y - car.getLocation_wc().y, aim.x - car.getLocation_wc().x);
		angle -= car.getOrientation_wc();
		angle = fmod(angle, kTwoPi);
		if (angle > kPi)	angle -= kTwoPi;
		if (angle < -kPi)	angle += kTwoPi;
		Tuple norm = aim - car.getLocation_wc();
		if (fabs(angle) > kPiOn2) {
			car.mThrottle = -0.75;
			car.mSteerWheelAngle = car.getOrientation_wc() + kPi - atan2(norm.y,norm.x);
		}
		else {
			car.mThrottle = 0.75;
			car.mSteerWheelAngle = atan2(norm.y,norm.x) - car.getOrientation_wc();
		}
	}
	else if (car.getVelocity().x < -0.5) {
		// stop the reversing
		car.mThrottle = 0.0;
		car.mBrake = 1.0;
	}
	else {
		car.mBrake = 0.0;
		
		// Calculate steering angle
		
		list<Tuple>::const_iterator pathC = pathB;
		pathC++;
		if (pathC == pathEnd)
			pathC++;
		
		// Steer parallel to path
		Tuple line = *pathC - *pathB;
		car.mSteerWheelAngle = atan2(line.y,line.x) - car.getOrientation_wc();
		Tuple norm = pathClosestPoint - car.getLocation_wc();
		
		// Correct to steer towards centre of path
		
		if ((line^norm) < 0)
			car.mSteerWheelAngle -= norm.Length() * 0.1;
		else
			car.mSteerWheelAngle += norm.Length() * 0.1;
		
		//car.mSteerWheelAngle += (line^norm) / line.Length() / kPi * 0.1;
		
		list<Tuple>::const_iterator pathD = pathC;
		pathD++;
		if (pathD == pathEnd)
			pathD++;
		
		// Look ahead and steer around corner slightly in advance
		double pathAngle = line.Angle(*pathD - *pathC);
		double segmentEndDistance = (car.getLocation_wc() - *pathC).Length();
		car.mSteerWheelAngle += pathAngle / (segmentEndDistance + 0.5);
		
		if (fabs(pathAngle) > kPi)
			printf("yeah\n");
		
		// Calculate throttle
		
		car.mThrottle = 1.0;
		double temp = 2*fabs(pathAngle) * car.getVelocity().x/(segmentEndDistance + 1.0);
		if (temp > 1.0)
			car.mThrottle /= temp;
		
		// Test for being stuck
		
		if (car.getVelocity_wc().Length() < 0.5)
			stuckTime += dt;
		else
			stuckTime = 0.0;
	}
		
	// Bring into range of -pi .. pi
	car.mSteerWheelAngle = fmod(car.mSteerWheelAngle, kTwoPi);
	if (car.mSteerWheelAngle < -kPi)	car.mSteerWheelAngle += kTwoPi;
	if (car.mSteerWheelAngle > kPi)		car.mSteerWheelAngle -= kTwoPi;
	
	// Bound range of steering angles
	if (car.mSteerWheelAngle > kMaxSteerAngle)
		car.mSteerWheelAngle = kMaxSteerAngle;
	if (car.mSteerWheelAngle < -kMaxSteerAngle)
		car.mSteerWheelAngle = -kMaxSteerAngle;
}
