/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  ScPanel.cpp
 *  Scoobie
 *
 *  Created by Ruben Zilibowitz on 24/11/04.
 *
 */

#include "ScPanel.h"
#include "ScView.h"

#include <cassert>

//
// constructor - from ScRect
//
ScPanel::ScPanel(const ScRect& inBounds)
	: bounds(inBounds), visible(true)
{
}

//
// constructor - from width and height
//
ScPanel::ScPanel(ScFloat width, ScFloat height)
	: visible(true)
{
	bounds.left = 0;
	bounds.right = width;
	bounds.bottom = 0;
	bounds.top = height;
}

//
// draw - iterate through list drawing each panel. Sub panels
// are drawn *after*. This panel is not drawn if it is outside
// the currently active view rect.
//
void ScPanel::draw()
{
	assert(ScView::sActiveViewRect != NULL);
	
	if (visible)
		if (rectsOverlap(*ScView::sActiveViewRect, bounds))
			doDraw();
	
	ScComponent::draw();
}

//
// moveTo - move panel to location
//
void ScPanel::moveTo(ScFloat x, ScFloat y)
{
	bounds.right += x - bounds.left;
	bounds.top += y - bounds.bottom;
	bounds.left = x;
	bounds.bottom = y;
}

//
// moveBy - offset panel by deltas
//
void ScPanel::moveBy(ScFloat dx, ScFloat dy)
{
	bounds.left += dx;
	bounds.top += dy;
	bounds.right += dx;
	bounds.bottom += dy;
}

//
// rectsOverlap - utility function to check if two rects are overlapping
//
bool ScPanel::rectsOverlap(const ScRect& rectA, const ScRect& rectB)
{
	return (std::min(rectA.right, rectB.right) > std::min(rectA.left, rectB.left)) &&
			(std::min(rectA.top, rectB.top) > std::min(rectA.bottom, rectB.bottom));
}
