/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  ScView.cpp
 *  Scoobie
 *
 *  Created by Ruben Zilibowitz on 25/11/04.
 *
 */

#include "ScView.h"

#if defined(__APPLE__) && defined(__MACH__)
#include <OpenGL/gl.h>
//#include <OpenGL/glu.h>
#else
#include <GL/gl.h>
//#include <GL/glu.h>
#endif

ScRect const * ScView::sActiveViewRect = NULL;

//
// constructor
//
ScView::ScView(const ScPortRect& inScreen, const ScRect& inView, ScAnimation &inAnimation)
	: screen(inScreen), view(inView), animation(inAnimation)
{
}

//
// draw - sets up viewport and ortho before sub layers draw
//
void ScView::draw()
{
	glLoadIdentity();
	glViewport(screen.left, screen.bottom, screen.width, screen.height);
	//gluOrtho2D(view.left, view.right, view.bottom, view.top);
	glOrtho(view.left, view.right, view.bottom, view.top, -1.0, 1.0);
	
	sActiveViewRect = &view;
    animation.draw();
	sActiveViewRect = NULL;
}

//
// scrollTo - scrolls so the given location is centred without changing the zoom
//
void ScView::scrollTo(ScFloat centre_x, ScFloat centre_y)
{
	ScFloat size_x = view.right - view.left;
	ScFloat size_y = view.top - view.bottom;
	
	view.left = centre_x - size_x*0.5;
	view.bottom = centre_y - size_y*0.5;
	view.right = centre_x + size_x*0.5;
	view.top = centre_y + size_y*0.5;
}

//
// scrollBy - scrolls by the given amount
//
void ScView::scrollBy(ScFloat dx, ScFloat dy)
{
	view.left += dx;
	view.bottom += dy;
	view.right += dx;
	view.top += dy;
}

