/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Zilibowitz on 31/12/04.
 */

#ifndef VEHICLE_H
#define VEHICLE_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "RigidBody.h"
#include "Wheel.h"

// Vehicle class
// This sub-class of RigidBody simulates the dynamics of a vehicle.

class Vehicle : public RigidBody
{
friend class Wheel;

public:
					Vehicle(    double inMass,
                                double inInertia,
								
                                Tuple inStartLocation,
                                double inStartOrientation,
								
                                double inFrontAxel,
                                double inRearAxel,
                                double inAxelWidth,
                                double inPinInset,
								
                                double inFrontTyreRadius,
                                double inRearTyreRadius,
                                double inFrontWheelInertia,
                                double inRearWheelInertia,
								
								PacejkaCurve& inFrontWheelSlipRatioCurve,
								PacejkaCurve& inFrontWheelSlipAngleCurve,
								PacejkaCurve& inRearWheelSlipRatioCurve,
								PacejkaCurve& inRearWheelSlipAngleCurve,
								
                                const Geom& inGeom,
								
                                double inDrag,
                                double inRollingResistance);
                    
                    Vehicle(const Vehicle& copyVehicle, Tuple inStartLocation, double inStartOrientation);
	
	virtual void	compute_force_and_torque();
	
    virtual void    rk2_step(double dt);
		
	// user inputs - public data members
	
	double  mSteerWheelAngle;   // radians
	double  mThrottle;			// 0..1
	double  mBrake;				// 0..1
	
protected:    
	double          engine_torque_curve(double rpm);
	
	// vehicle design
	
	double		mFrontAxel;
	double		mRearAxel;
	double		mWheelbase;     // mFrontAxel + mRearAxel
	double		mAxelWidth;
    double		mDragConstant;
	double		mEngineBrake;	// braking torque of unpowered engine
	
    // wheels
    
    Wheel   mWheel_FL;
    Wheel   mWheel_FR;
    Wheel   mWheel_BL;
    Wheel   mWheel_BR;
    
	// engine
	
	double	mRPM;
	double	mDriveTorque;
	
    // working data
    
	Tuple   mVelocity;
	Tuple   mDrag;
	Tuple   mTotalForce;
	
public:
	// accessor functions
	
	const Wheel& getWheel_FL() const { return mWheel_FL; }
	const Wheel& getWheel_FR() const { return mWheel_FR; }
	const Wheel& getWheel_BL() const { return mWheel_BL; }
	const Wheel& getWheel_BR() const { return mWheel_BR; }
	
	Tuple getVelocity() const { return mVelocity; }
	Tuple getDrag() const { return mDrag; }
	Tuple getTotalForce() const { return mTotalForce; }
	double getRPM() const { return mRPM; }
	
	// kill
	
	void kill();
};

#endif
