/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Zilibowitz on 31/12/04.
 */

#ifndef WHEEL_H
#define WHEEL_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "Tuple.h"
#include "PacejkaCurve.h"

// forwards declaration
class Vehicle;

// class Wheel

class Wheel
{
friend class Vehicle;

public:
	Wheel(  Vehicle& inParentVehicle,
			PacejkaCurve& inSlipRatioCurve,
			PacejkaCurve& inSlipAngleCurve,
			Tuple inHinge_vc,
			double inRadius,
			double inTurnRadius,
			double inOrientation_vc,
			double inInertia,
			double inMaxBrakeTorque,
			double inRollingResistance);
	
    Wheel(  Vehicle& inParentVehicle,
            const Wheel& inWheel);
    
	void setOrientation(double inOrientation_vc);
	void compute_force_and_torque();
	
protected:
	Vehicle&	mVehicle;
	PacejkaCurve&	mSlipRatioCurve;
	PacejkaCurve&	mSlipAngleCurve;
	
	Tuple   mHinge_vc;      // location of joint to vehicle
	double  mWheelRadius;   // in xz plane (distance from hub to contact patch)
	double  mJointRadius;   // in xy plane (distance from hinge joint to contact patch)
	
	double  mOrientation_vc;    // vehicle coords
	Tuple   mNorm;              // vehicle coords
	Tuple   mLocation_vc;       // vehicle coords = mHinge_vc + mJointRadius*mNorm
	
	double  mLoad;          // the amount of weight on this wheel (N)
	double  mInertia;       // the inertia rotating in the xz plane around its hub
	double  mInvInertia;    // reciprocal of inertia (for faster computations)
	
	double  mBrake;             // the braking strength (0..1)
	double	mMaxBrakeTorque;	// maximum braking torque
	double  mRollingResistance; // the rolling resistance
	
	Tuple   mVelocity;          // wheel coords
	Tuple   mForce;             // wheel coords
	Tuple   mForce_vc;          // vehicle coords
	
	double  mTractionTorque;        // in xz plane (Nm)
	double	mTotalTorque;			// in xz plane (Nm)
	double  mVehicleTorque;         // in xy plane (Nm)
	
	double	mTangentVelocity;		// velocity of edge of wheel (m/s)
	double  mAngularVelocity;       // in xz plane (around y) (rad/s)
	double  mAngularAcceleration;   // in xz plane (around y) (rad/s/s)
	
	double  mSlipRatio;
	double  mLongtitudinalSlipVelocity;
	double  mSlipAngle;
	
	Tuple	mSlipVector;			// normalised slip
	double	mTotalSlip;				// size of normalised slip vector
	
	double	mPeakSlipRatio;
	double	mPeakSlipAngle;
	
public:
	// accessor functions
	
	const Vehicle&	getVehicle() const { return mVehicle; }
	
	Tuple   getHinge_vc() const { return mHinge_vc; }
	//double  mWheelRadius;
	//double  mJointRadius;
	
	double  getOrientation_vc() const { return mOrientation_vc; }
	Tuple   getNorm() const { return mNorm; }
	Tuple	getLocation_vc() const { return mLocation_vc; }
	/*
	double  mLoad;
	double  mInertia;
	double  mInvInertia;
	
	double  mBrake;
	double	mMaxBrakeTorque;
	double  mRollingResistance;
	*/
	Tuple   getVelocity() const { return mVelocity; }
	/*
	Tuple   mForce;
	Tuple   mForce_vc;
	
	double  mTractionTorque;
	double	mTotalTorque;
	double  mVehicleTorque;         // in xy plane (Nm)
	
	double	mTangentVelocity;		// velocity of edge of wheel (m/s)
	double  mAngularVelocity;       // in xz plane (around y) (rad/s)
	double  mAngularAcceleration;   // in xz plane (around y) (rad/s/s)
	
	double  mSlipAngle;
	double  mSlipRatio;
	*/
	double  getLongtitudinalSlipVelocity() const { return mLongtitudinalSlipVelocity; }
	
	/*
	Tuple	mSlipVector;			// normalised slip
	double	mTotalSlip;				// size of normalised slip vector
	
	double  slip_angle_friction(double alpha);
	double  slip_ratio_friction(double ratio);
	
	double	mPeakSlipAngle;
	double	mPeakSlipRatio;
	*/
};

#endif
